@@ -393,13 +393,14 @@ def generate_standard_asimov(**kwargs):
     fix_param = kwargs.pop('fix_param')
     profile_param = kwargs.pop('profile_param')
     snapshot_name = kwargs.pop('snapshot_name')
+    poi_scale = kwargs.pop("poi_scale")
     config = {
         'fix_param': fix_param,
         'profile_param': profile_param,
         'snapshot_name': snapshot_name
     }
     generator = AsimovGenerator(**kwargs, config=config)
-    generator.generate(asimov_types)
+    generator.generate_standard_asimov(asimov_types, poi_scale=poi_scale)
     generator.save(outname)
     
 
