@@ -3,9 +3,6 @@ import enum
 
 from quickstats.components import AnalysisObject
 from quickstats.utils.common_utils import parse_config
-from quickstats.utils.io import VerbosePrint
-
-_PRINT_ = VerbosePrint("INFO")
 
 class AsimovType(enum.Enum):
     S_NP_Nom               = -2
@@ -91,45 +88,51 @@ class AsimovGenerator(AnalysisObject):
     
     DEFAULT_ASIMOV_TYPES = [AsimovType.B_NP_Fit, AsimovType.S_NP_Fit, AsimovType.S_NP_Fit_muhat]
     
-    def __init__(self, filename:str, poi_name:str=None, 
-                 poi_scale:Optional[float]=None,
+    def __init__(self, filename:str, poi_name:str=None,
                  data_name:str='combData', 
                  config:Optional[Union[Dict, str]]=None,
-                 verbosity:Optional[Union[int, str]]=None):
-        
+                 verbosity:Optional[Union[int, str]]="INFO"):
         config = parse_config(config)
         config['filename']  = filename
+        config['poi_name']  = poi_name
         config['data_name'] = data_name
-        if verbosity is not None:
-            _PRINT_.verbosity = verbosity
-            config['verbosity'] = verbosity
-        super().__init__(**config)
-        
-        if poi_name is None:
-            poi = self.model.pois.first()
-            if not poi:
-                raise RuntimeError("No POI found in the workspace")
-            _PRINT_.info(f"INFO: POI not specified. The first POI `{poi.GetName()}` will be used.")
-            
-        self.poi_name = poi_name
-        self.poi_scale = poi_scale
+        config['verbosity'] = verbosity
+        self._inherit_init(super(AsimovGenerator, self).__init__, **config)
         
     
-    def generate(self, asimov_types:Optional[Union[List[AsimovType], List[int]]]=None):
+    def generate_standard_asimov(self, asimov_types:Optional[Union[List[AsimovType], List[int]]]=None,
+                                 asimov_names:Optional[Union[List[str], str]]=None,
+                                 asimov_snapshots:Optional[Union[List[str], str]]=None,
+                                 poi_scale:Optional[float]=None):
         if asimov_types is None:
             asimov_types = self.DEFAULT_ASIMOV_TYPES
-        for asimov_type in asimov_types:
+        if isinstance(asimov_types, AsimovType):
+            asimov_types = [asimov_types]
+        if asimov_names is not None:
+            if isinstance(asimov_names, str):
+                asimov_names = [asimov_names]
+            assert len(asimov_names) == len(asimov_types)
+        if asimov_snapshots is not None:
+            if isinstance(asimov_snapshots, str):
+                asimov_snapshots = [asimov_snapshots]
+            assert len(asimov_snapshots) == len(asimov_types)            
+        if poi_scale is None:
+            poi_scale = 1.0
+        poi_name = self.poi.GetName()
+        self.model.workspace.saveSnapshot("temp", self.model.workspace.allVars())
+        for i, asimov_type in enumerate(asimov_types):
+            self.load_snapshot("temp")
             if isinstance(asimov_type, int):
                 asimov_type = AsimovType(asimov_type)
             kwargs = self.ASIMOV_SETTINGS[asimov_type]
-            kwargs['poi_name'] = self.poi_name
+            kwargs['poi_name'] = poi_name
             for key in ['poi_val', 'poi_profile']:
                 if kwargs[key] is not None:
-                    kwargs[key] *= self.poi_scale
+                    kwargs[key] *= poi_scale
+            if asimov_names is not None:
+                kwargs['asimov_name'] = asimov_names[i]
+            if asimov_snapshots is not None:
+                kwargs['asimov_snapshot'] = asimov_snapshots[i]
             kwargs['minimizer_options'] = self.minimizer_options
             kwargs['nll_options'] = self.nll_commands
-            self.model.generate_asimov(**kwargs)
-            
-    def save(self, fname:str, recreate:bool=True):
-        self.model.save(fname, recreate=recreate)
-        _PRINT_.info(f"INFO: Saved workspace file as `{fname}`")
\ No newline at end of file
+            self.model.generate_asimov(**kwargs)
\ No newline at end of file
