@@ -48,6 +48,7 @@ class AsymptoticCLs(AnalysisObject):
         config = {
             'filename': filename,
             'data_name': data_name,
+            'poi_name': poi_name,
             'binned_likelihood' : binned_likelihood,
             'fix_param': fix_param,
             'profile_param': profile_param,
@@ -86,7 +87,6 @@ class AsymptoticCLs(AnalysisObject):
         self.direction = 1
         self.global_status = 0
         
-        self.set_poi(poi_name)
         # define simplified names
         self.ws = self.model.workspace
         self.globs = self.model.global_observables
