@@ -5,6 +5,7 @@
 ##################################################################################################
 import os
 import sys
+import copy
 import math
 import logging
 import fnmatch
@@ -100,15 +101,15 @@ class ExtendedModel(AbstractObject):
         elif isinstance(val, list):
             self._initial_snapshots = val
         else:
-            raise ValueError("`initial_snapshots` must be string or list of strings")
+            raise ValueError("\"initial_snapshots\" must be string or list of strings")
     
     @semistaticmethod
     def load_extension(self):
         try:
-            if not hasattr(ROOT, 'RooTwoSidedCBShape'):
-                result = load_macro('RooTwoSidedCBShape')
-                if hasattr(ROOT, 'RooTwoSidedCBShape'):
-                    self.stdout.info('INFO: Loaded extension module "RooTwoSidedCBShape"')
+            for extension in ['RooTwoSidedCBShape', 'FlexibleInterpVarMkII']:
+                result = load_macro(extension)
+                if (result is not None) and hasattr(ROOT, extension):
+                    self.stdout.info(f'INFO: Loaded extension module "{extension}"')    
         except Exception as e:
             print(e)
             
@@ -293,23 +294,23 @@ class ExtendedModel(AbstractObject):
         self.remove_msg_topic()
         return None
                 
-    @staticmethod
-    def _fix_parameters(source:"ROOT.RooArgSet", param_expr=None, param_str='parameter'):
+    @semistaticmethod
+    def _fix_parameters(self, source:"ROOT.RooArgSet", param_expr=None, param_str='parameter'):
         '''
             source: parameters instance
             param_expr: 
         '''            
-        param_dict = ExtendedModel.parse_param_expr(param_expr)
-        return ExtendedModel._set_parameters(source, param_dict, mode='fix', param_str=param_str)           
+        param_dict = self.parse_param_expr(param_expr)
+        return self._set_parameters(source, param_dict, mode='fix', param_str=param_str)           
     
-    @staticmethod
-    def _profile_parameters(source:"ROOT.RooArgSet", param_expr=None, param_str='parameter'):
+    @semistaticmethod
+    def _profile_parameters(self, source:"ROOT.RooArgSet", param_expr=None, param_str='parameter'):
         '''
             source: parameters instance
             param_expr: 
         '''                          
-        param_dict = ExtendedModel.parse_param_expr(param_expr)
-        return ExtendedModel._set_parameters(source, param_dict, mode='free', param_str=param_str)   
+        param_dict = self.parse_param_expr(param_expr)
+        return self._set_parameters(source, param_dict, mode='free', param_str=param_str)   
     
     def fix_parameters(self, param_expr=None):
         return self._fix_parameters(self.workspace.allVars(), param_expr=param_expr,
@@ -341,7 +342,7 @@ class ExtendedModel(AbstractObject):
             if not selected_params:
                 self.stdout.warning('WARNING: Parameter "{}" does not exist. No modification will be made.'.format(name))
             for param_name in selected_params:
-                ExtendedModel._set_parameter(source[param_name], param_dict[name], mode=mode, param_str=param_str)
+                self._set_parameter(source[param_name], param_dict[name], mode=mode, param_str=param_str)
                 set_parameters.append(source[param_name])
 
         return set_parameters
@@ -621,7 +622,7 @@ class ExtendedModel(AbstractObject):
             elif pdf_class == "RooPoisson":
                 nuis.setVal(1)
             else:
-                raise RuntimeError(f"constraint term `{pdf.GetName()}` has unsupported type `{pdf_class}`")
+                raise RuntimeError(f"constraint term \"{pdf.GetName()}\" has unsupported type \"{pdf_class}\"")
         if set_constant:
             for nuis in constrained_nuis:
                 nuis.setConstant(1)
@@ -715,13 +716,13 @@ class ExtendedModel(AbstractObject):
     def get_poi(self, poi_name:Optional[str]=None, strict:bool=False):
         if poi_name is None:
             poi = self.pois.first()
-            self.stdout.info(f"INFO: POI name not specified. The first POI `{poi.GetName()}` is used by default.")
+            self.stdout.info(f"INFO: POI name not specified. The first POI \"{poi.GetName()}\" is used by default.")
         else:
             poi = self.workspace.var(poi_name)
         if not poi:
-            raise RuntimeError(f"workspace does not contain the variable `{poi_name}`")
+            raise RuntimeError(f"workspace does not contain the variable \"{poi_name}\"")
         if strict and (poi not in list(self.pois)):
-            raise RuntimeError(f"workspace variable `{poi_name}` is not part of the POIs")
+            raise RuntimeError(f"workspace variable \"{poi_name}\" is not part of the POIs")
         return poi
     
     def _load_obs_and_weight(self, obs_and_weight:Optional[Union["ROOT.RooArgSet",str]]=None, 
@@ -761,7 +762,7 @@ class ExtendedModel(AbstractObject):
     def get_object_map(object_dict:Dict, object_name:str):
         ExtendedModel.load_RooFitObjects()
         if object_name not in ["RooDataSet", "RooAbsPdf"]:
-            raise ValueError("unsupported object `{}`".format(object_name))
+            raise ValueError("unsupported object \"{}\"".format(object_name))
         object_map = ROOT.std.map(f"string, {object_name}*")()
         object_map.keepalive = list()
         for c, d in object_dict.items():
@@ -850,7 +851,7 @@ class ExtendedModel(AbstractObject):
                 set_trace()                
                 globs.setVal(nuis.getVal()*globs.getVal())
             else:
-                raise RuntimeError(f"constraint term `{pdf.GetName()}` has unsupported type `{pdf_class}`")
+                raise RuntimeError(f"constraint term \"{pdf.GetName()}\" has unsupported type \"{pdf_class}\"")
             self.stdout.debug("set glob {} to val {}".format(globs.GetName(), globs.getVal()))
         """
 
@@ -937,7 +938,7 @@ class ExtendedModel(AbstractObject):
         all_nuis  = self.nuisance_parameters
         
         # define names used for various objects
-        names = self._DEFAULT_NAMES_
+        names = copy.deepcopy(self._DEFAULT_NAMES_)
         if snapshot_names is not None:
             names.update(snapshot_names)
         if asimov_name is not None:
@@ -967,9 +968,9 @@ class ExtendedModel(AbstractObject):
                 dataset = self.data
             minimizer = ExtendedMinimizer("Minimizer", self.pdf, dataset)
             if minimizer_options is None:
-                minimizer_options = minimizer._DEFAULT_MINIMIZER_OPTION_
+                minimizer_options = copy.deepcopy(minimizer._DEFAULT_MINIMIZER_OPTION_)
             if nll_options is None:
-                nll_options = minimizer._DEFAULT_NLL_OPTION_
+                nll_options = copy.deepcopy(minimizer._DEFAULT_NLL_OPTION_)
             minimizer.configure(**minimizer_options)
             if constraint_option == 0:
                 pass
@@ -1007,7 +1008,7 @@ class ExtendedModel(AbstractObject):
         else:
             ws.saveSnapshot(con_glob_name.format(mu=pretty_value(poi_val)), all_globs)
             ws.saveSnapshot(con_nuis_name.format(mu=pretty_value(poi_val)), all_nuis)
-        
+
         asimov_data_name = names['asimov'].format(mu=pretty_value(poi_val), 
                                                   mu_cond=pretty_value(poi_profile))
         channel_asimov_data_name = names['channel_asimov']
@@ -1034,7 +1035,7 @@ class ExtendedModel(AbstractObject):
                                           ROOT.RooFit.WeightVar(weight_var))
         if do_import:
             getattr(ws, "import")(asimov_data)
-            self.stdout.info(f'INFO: Generated Asimov Dataset `{asimov_data_name}`')
+            self.stdout.info(f'INFO: Generated Asimov Dataset \"{asimov_data_name}\"')
             
         if asimov_snapshot is not None:
             self.stdout.info('INFO: Saving snapshot "{}"'.format(asimov_snapshot))
@@ -1049,7 +1050,7 @@ class ExtendedModel(AbstractObject):
         elif restore_states == 2:
             pass
         else:
-            raise ValueError(f"unsupported restore state option `{restore_states}`")
+            raise ValueError(f"unsupported restore state option \"{restore_states}\"")
         
         return asimov_data
     
@@ -1185,7 +1186,7 @@ class ExtendedModel(AbstractObject):
                 else:
                     raise RuntimeError("multiple RooCategory instances found in the dataset")
             else:
-                raise RuntimeError(f"unknown object type `{type(p)}` found in the dataset")
+                raise RuntimeError(f"unknown object type \"{type(p)}\" found in the dataset")
         n_entries = dataset.numEntries()
         result = {}
         """
@@ -1223,7 +1224,7 @@ class ExtendedModel(AbstractObject):
             distributions[cat] = {}
             observable = category_map[cat]['observable']
             if observable not in dataset_values:
-                raise RuntimeError(f"no data associated with the observable `{observable}` found in the dataset")
+                raise RuntimeError(f"no data associated with the observable \"{observable}\" found in the dataset")
             distributions[cat]['observable'] = observable
             mask = (dataset_values['label'] == cat)
             x = dataset_values[observable][mask]
@@ -1289,7 +1290,7 @@ class ExtendedModel(AbstractObject):
                     dataset_name = dataset
                     dataset = self.workspace.data(dataset)
                     if not dataset:
-                        raise ValueError(f"dataset `{dataset_name}` does not exist")
+                        raise ValueError(f"dataset \"{dataset_name}\" does not exist")
                 dataset_name = dataset.GetName()
                 distributions = self.get_dataset_distributions(dataset)
                 for category in distributions:
@@ -1302,7 +1303,7 @@ class ExtendedModel(AbstractObject):
                 for snapshot in snapshots:
                     exist = self.workspace.loadSnapshot(snapshot)
                     if not exist:
-                        raise RuntimeError("snapshot `{}` not found in workspace".format(snapshot))
+                        raise RuntimeError("snapshot \"{}\" not found in workspace".format(snapshot))
                     distributions = self.get_distributions()
                     for category in distributions:
                         if category not in collected_distributions:
@@ -1318,7 +1319,7 @@ class ExtendedModel(AbstractObject):
         from quickstats.plots.template import plot_distributions
         category_map = self.get_category_map()
         if category not in category_map:
-            raise ValueError("category `{}` not found in workspace".format(category))
+            raise ValueError("category \"{}\" not found in workspace".format(category))
         collected_distributions = self.get_collected_distributions(current_distributions,
                                                                    datasets, snapshots=snapshots)
         xlabel = category_map[category]['observable']
@@ -1349,7 +1350,8 @@ class ExtendedModel(AbstractObject):
                       include_patterns:Optional[List]=None, exclude_patterns:Optional[List]=None,
                       save_as:Optional[str]=None):
         if items is None:
-            items = ['workspace', 'dataset', 'category', 'poi', 'detailed_nuisance_parameter']
+            items = ['workspace', 'dataset', 'category', 'observable', 
+                     'poi', 'detailed_nuisance_parameter']
         summary_str = ""
         # workspace
         if 'workspace' in items:
@@ -1359,7 +1361,7 @@ class ExtendedModel(AbstractObject):
         if 'dataset' in items:
             datasets = self.workspace.allData()
             summary_str += f"Datasets ({len(datasets)}):\n"
-            summary_str += "\t" + ", ".join([d.GetName() for d in datasets]) + "\n"
+            summary_str += "".join([f"\t{obs.GetName()}\n" for obs in datasets])
         # categories
         if 'category' in items:
             if isinstance(self.pdf, ROOT.RooSimultaneous):
@@ -1370,6 +1372,10 @@ class ExtendedModel(AbstractObject):
                     channel_cat.setIndex(i)
                     label = channel_cat.getLabel()
                     summary_str += f"\t{label}\n"
+        # observables
+        if 'observable' in items:
+            summary_str += f"Observables ({len(self.observables)}):\n"
+            summary_str += "".join([f"\t{obs.GetName()}\n" for obs in self.observables])
         # pois, NPs
         param_strs     = []
         param_sets     = []
