@@ -177,71 +177,87 @@ class Likelihood(AnalysisObject):
         return self.minimizer.nll.getVal()
     
     @property
-    def status(self):
+    def fit_status(self):
         return self.minimizer.status
     
     def __init__(self, filename:str, poi_name:Optional[str]=None,
                  data_name:str='combData', 
-                 config:Optional[Union[Dict, str]]=None):
-        
+                 config:Optional[Union[Dict, str]]=None,
+                 verbosity:Optional[Union[int, str]]="INFO"):
+
         config = parse_config(config)
         config['filename']  = filename
+        config['poi_name']  = poi_name
         config['data_name'] = data_name
-        super().__init__(**config)
+        config['verbosity'] = verbosity
+        self._inherit_init(super().__init__, **config)
         
-        self.model.workspace.saveSnapshot("initialSnapshot", self.model.workspace.allVars())
-        self.poi = self.model.get_poi(poi_name)
+        self.model.workspace.saveSnapshot("initialSnapshot", self.model.workspace.allVars())        
         
-    def _evaluate(self, poi_val:Optional[float]=None, 
-                  snapshot_name:Optional[str]="initialSnapshot"):
-        if snapshot_name:
-            self.model.workspace.loadSnapshot(snapshot_name)
+    def evaluate_nll(self, poi_val:Optional[float]=None, mode:int=2,
+                     snapshot_name:Optional[str]="initialSnapshot"):
+        """Evalute post-fit NLL value
+        
+        Arguments:
+            poi_val: (Optional) float
+                Value of parameter of interest during fit. If not specified, the original value is used.
+            mode: int
+                Evaluation mode. Choose from:
+                    0: Evaluate nll_mu - nll_muhat
+                    1: Evaluate unconditional NLL (nll_muhat)
+                    2: Evaluate conditional NLL (nll_mu)
+            snapshot_name: (Optional) str
+                Name of snapshot to load before fitting.
+        """
         if poi_val is None:
+            poi_val = self.poi.getVal()
+        
+        nll_muhat = None
+        nll_mu    = None
+        muhat     = None
+        start_time = time.time()
+        # unconditional nll
+        if mode in [0, 1]:
+            self.load_snapshot(snapshot_name)
             self.poi.setConstant(0)
-        else:
             self.poi.setVal(poi_val)
+            self.minimizer.minimize()
+            muhat     = self.poi.getVal()
+            nll_muhat = self.minimizer.nll.getVal()
+        fit_time_uncond = time.time() - start_time
+        # conditional nll
+        if mode in [0, 2]:
+            self.load_snapshot(snapshot_name)
             self.poi.setConstant(1)
-        self.minimizer.minimize()
-        
-        return self.minimizer.nll.getVal()
-        
-    def evaluate(self, poi_val:float, vmin:float=10., vmax:float=10., 
-                 unconditional=False, hesse:bool=False, minos:bool=False, 
-                 snapshot_name:Optional[str]="initialSnapshot"):
-        
-        if snapshot_name:
-            self.model.workspace.loadSnapshot(snapshot_name)
-            
-        # setup the POI
-        poi_min, poi_max = poi_val - abs(vmin), poi_val + abs(vmax)
-        self.poi.setRange(poi_min, poi_max)
-        
-        if unconditional:
-            self.poi.setConstant(0)
-        else:
             self.poi.setVal(poi_val)
-            self.poi.setConstant(1)
-
-        minimizer_options = {
-            'hesse': hesse,
-            'minos': minos
-        }
+            self.minimizer.minimize()
+            nll_mu = self.minimizer.nll.getVal()
+        fit_time_cond = time.time() - fit_time_uncond - start_time
         
-        if minos:
-            minimizer_options['minos_set'] = ROOT.RooArgSet(self.poi)
-
-        # perform the fit
-        self.minimizer.minimize(**minimizer_options)
+        self.stdout.info("INFO: NLL evaluation completed with")
+        if mode in [0, 1]:
+            self.stdout.info("\tmu hat = {:.5f}\n\tuncond NLL = {:.5f}".format(muhat, nll_muhat))
+        if mode in [0, 2]:
+            self.stdout.info("\tmu = {:.5f}".format(poi_val))
+            self.stdout.info("\tcond NLL = {:.5f}".format(nll_mu))
+        if mode == 0:
+            self.stdout.info("\tPNLL = {:.5f}".format(nll_mu - nll_muhat))
+        self.stdout.info("\ttime (uncond_fit, cond_fit) = {:.3f}, {:.3f}".format(fit_time_uncond, fit_time_cond))
         
-        return self.minimizer.nll.getVal()
+        if mode == 0:
+            return nll_mu - nll_muhat
+        if mode == 1:
+            return nll_muhat
+        if mode == 2:
+            return nll_mu
     
-def evaluate_nll(filename:str, poi_val:Optional[float]=None, poi_name:str=None,
+def evaluate_nll(filename:str, poi_val:Optional[float]=None, poi_name:str=None, mode:int=1,
                  ws_name:Optional[str]=None, mc_name:Optional[str]=None, data_name:str='combData', 
                  snapshot_name:str=None, profile_param:str="", fix_param:str="", 
                  constrain_nuis:bool=True, minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
                  num_cpu:int=1, binned_likelihood:bool=True, eps:float=1.0, strategy:int=1, 
-                 optimize:int=2, offset:bool=False, fix_cache:bool=True, fix_multi:bool=True, 
-                 max_calls:int=-1, max_iters:int=-1, print_level:int=1, verbosity:int="INFO",  
+                 optimize:int=2, offset:bool=True, fix_cache:bool=True, fix_multi:bool=True, 
+                 max_calls:int=-1, max_iters:int=-1, print_level:int=-1, verbosity:int="WARNING",  
                  outname:str=None, cache:bool=False, detailed_output:bool=False):
     if cache:
         cached_result = load_cache(outname)
@@ -253,7 +269,7 @@ def evaluate_nll(filename:str, poi_val:Optional[float]=None, poi_name:str=None,
             else:
                 print('INFO: Cached NLL for POI "{}" at {:.2f}: {}'.format(poi_name, poi_val, nll))
             return nll
-    start = time.time()    
+    
     config = {
         'ws_name' : ws_name,
         'mc_name' : mc_name,
@@ -275,20 +291,21 @@ def evaluate_nll(filename:str, poi_val:Optional[float]=None, poi_name:str=None,
         'max_iters': max_iters,
         'print_level': print_level,
         'verbosity': verbosity,
-    }    
-    likelihood = Likelihood(filename=filename, poi_name=poi_name, data_name=data_name, config=config)
-    poi_name = likelihood.poi.GetName()
-    nll = likelihood._evaluate(poi_val)
-    end = time.time()
+    }
     if poi_val is None:
-        print('INFO: Unconditional NLL for POI "{}": {}'.format(poi_name, nll))
+        print(f'INFO: Evaluating unconditional NLL for POI "{poi_name}"')
     else:
-        print('INFO: NLL for POI "{}" at {:.2f}: {}'.format(poi_name, poi_val, nll))
-    print('INFO: Time taken to evaluate NLL: {:.3f} s'.format(end-start))
-
+        print(f'INFO: Evaluating conditional NLL for POI "{poi_name}" at mu = {poi_val}')
+    start = time.time()
+    likelihood = Likelihood(filename=filename, poi_name=poi_name, data_name=data_name, config=config,
+                            verbosity=verbosity)
+    poi_name = likelihood.poi.GetName()
+    nll = likelihood.evaluate_nll(poi_val, mode=mode)
+    end = time.time()
     results = {
         'nll': nll,
         'poi': poi_name,
+        'offset': int(offset),
         'constrain_nuis': int(constrain_nuis),
         'poi_value': poi_val
     }
@@ -312,11 +329,12 @@ def scan_nll(filename:str, poi_min:float, poi_max:float, poi_step:float, poi_nam
              mc_name:Optional[str]=None, data_name:str='combData', snapshot_name:str=None, 
              profile_param:str="", fix_param:str="", constrain_nuis:bool=True,
              minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned_likelihood:bool=True,
-             eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, verbosity:int="INFO", 
-             print_level:int=1, max_calls:int=-1, max_iters:int=-1, optimize:int=2, offset:bool=True, parallel:int=-1):
+             eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, verbosity:int="WARNING", 
+             print_level:int=-1, max_calls:int=-1, max_iters:int=-1, optimize:int=2, offset:bool=True, parallel:int=-1):
     start_time = time.time()
-    points        = np.arange(poi_min, poi_max+poi_step, poi_step)
-    points        = np.concatenate(([None], points))
+    points     = np.arange(poi_min, poi_max+poi_step, poi_step)
+    points     = np.concatenate(([None], points))
+    modes      = [1] + [2]*(len(points)-1)
     # try to get the number of significant digits of poi_step
     sd = approx_n_digit(poi_step)
     if poi_name is None:
@@ -332,7 +350,7 @@ def scan_nll(filename:str, poi_min:float, poi_max:float, poi_step:float, poi_nam
     if not os.path.exists(cachedir):
         os.makedirs(cachedir)
 
-    arguments = (repeat(filename), points, repeat(poi_name), repeat(ws_name),repeat(mc_name), 
+    arguments = (repeat(filename), points, repeat(poi_name), modes, repeat(ws_name),repeat(mc_name), 
                  repeat(data_name), repeat(snapshot_name), repeat(profile_param), repeat(fix_param), 
                  repeat(constrain_nuis), repeat(minimizer_type), repeat(minimizer_algo), repeat(num_cpu),
                  repeat(binned_likelihood), repeat(eps), repeat(strategy), repeat(optimize), repeat(offset),
@@ -351,4 +369,4 @@ def scan_nll(filename:str, poi_min:float, poi_max:float, poi_step:float, poi_nam
     with open(final_outname, 'w') as outfile:
         json.dump(data, outfile, indent=3)
     end_time = time.time()
-    print('INFO: All jobs have finished. Total Time taken: {:.3f} s'.format(end_time-start_time))  
\ No newline at end of file
+    print('INFO: All jobs have finished. Total time taken: {:.3f} s'.format(end_time-start_time))  
\ No newline at end of file
