@@ -1,4 +1,4 @@
-from typing import Optional, Union, Dict, List
+from typing import Optional, Union, Dict, List, Tuple
 from copy import deepcopy
 import re
 import numpy as np
@@ -7,7 +7,8 @@ import matplotlib.transforms as transforms
 from matplotlib.patches import Rectangle
 from matplotlib.lines import Line2D
 from matplotlib.ticker import (MultipleLocator, FormatStrFormatter,
-                               AutoMinorLocator, ScalarFormatter)
+                               AutoMinorLocator, ScalarFormatter,
+                               Locator, Formatter, AutoLocator)
 
 from quickstats.utils.common_utils import update_nested_dict
 
@@ -38,10 +39,6 @@ TEMPLATE_STYLES = {
             'dpi': 72,
             'facecolor': "#FFFFFF"
         },
-        'strip_extra_axis_label_decimals': {
-            'xaxis': True,
-            'yaxis': True
-        },
         'legend_Line2D': {
             'linewidth': 3
         },
@@ -57,6 +54,16 @@ TEMPLATE_STYLES = {
             'labelsize': 20,
             'tick_bothsides': True
         },
+        'xtick':{
+            'format': 'numeric',
+            'steps': None,
+            'prune': None
+        },
+        'ytick':{
+            'format': 'numeric', 
+            'steps': None,
+            'prune': None
+        },        
         'xlabel': {
             'fontsize': 22,
             'loc' : 'right',
@@ -70,6 +77,9 @@ TEMPLATE_STYLES = {
         'text':{
             'fontsize': 20,
         },
+        'plot':{
+            'linewidth': 2
+        },
         'errorbar': {
             "marker": 'x',
             "linewidth": 0,
@@ -167,11 +177,8 @@ def single_frame(logx:bool=False, logy:bool=False,
         ax.set_xscale('log')
     if logy:
         ax.set_yscale('log')
-    format_axis_ticks(ax, x_axis=True, y_axis=True, **styles['axis'])
-    if (styles['strip_extra_axis_label_decimals']['xaxis']) and (not logx):
-        ax.xaxis.set_major_formatter(NumericFormatter())
-    if (styles['strip_extra_axis_label_decimals']['yaxis']) and (not logy):
-        ax.yaxis.set_major_formatter(NumericFormatter())
+    format_axis_ticks(ax, x_axis=True, y_axis=True, xtick_styles=styles['xtick'],
+                      ytick_styles=styles['ytick'], **styles['axis'])
         
     if analysis_label_options is not None:
         draw_atlas_label(ax, dy=0.05, text_options=styles['text'], **analysis_label_options)
@@ -189,9 +196,11 @@ def suggest_markersize(nbins:int):
 def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length:int=16, minor_length:int=8,
                       spine_width:int=2, major_width:int=2, minor_width:int=1, 
                       label_bothsides:bool=False, tick_bothsides:bool=False,
-                      labelsize:Optional[int]=None, 
+                      labelsize:Optional[int]=None,
                       x_axis_styles:Optional[Dict]=None, 
-                      y_axis_styles:Optional[Dict]=None):
+                      y_axis_styles:Optional[Dict]=None,
+                      xtick_styles:Optional[Dict]=None,
+                      ytick_styles:Optional[Dict]=None):
     if x_axis:
         if (ax.get_xaxis().get_scale() != 'log'):
             ax.xaxis.set_minor_locator(AutoMinorLocator())
@@ -222,6 +231,33 @@ def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length:int=16, minor_l
                        width=minor_width, **styles)
     for axis in ['top','bottom','left','right']:
         ax.spines[axis].set_linewidth(spine_width)
+        
+    set_axis_tick_styles(ax.xaxis, xtick_styles)
+    set_axis_tick_styles(ax.yaxis, ytick_styles)
+
+def set_axis_tick_styles(ax, styles=None):
+    if styles is None:
+        return None
+    fmt = styles['format']
+    if fmt is not None:
+        formatter = None
+        if isinstance(fmt, str):
+            if fmt == 'numeric':
+                formatter = NumericFormatter()
+        if isinstance(fmt, Formatter):
+            formatter = fmt
+        if formatter is None:
+            raise ValueError(f"unsupported axis tick format {fmt}")
+        ax.set_major_formatter(formatter)
+    locator = AutoLocator()
+    locator_params = {}
+    for param in ['steps', 'prune']:
+        value = styles[param]
+        if value is not None:
+            locator_params[param] = value
+    locator.set_params(**locator_params)
+    ax.set_major_locator(locator)
+                
 
 def centralize_axis(ax, which='x'):
     if which == 'x':
@@ -288,6 +324,7 @@ def draw_text(axis, x, y, s, transform_x:str='axis',
     xmin, xmax, ymin, ymax = get_box_dimension(text)
     return xmin, xmax, ymin, ymax
 
+
 def draw_ATLAS_label(axis, x=0.05, y=0.95, fontsize=25, extra='Internal', 
                      transform_x='axis', transform_y='axis', 
                      vertical_align='top', horizontal_align='left'):
@@ -331,7 +368,7 @@ def draw_atlas_label(axis, x=0.05, y=0.95, fontsize=25, status:str='int',
     else:
         status_str = status
         
-    xmin, xmax, ymin, ymax = draw_ATLAS_label(axis, x, y, fontsize, extra=status_str,
+    xmin, xmax, ymin, ymax = draw_ATLAS_label(axis, x, y, fontsize, extra=" "+status_str,
                                               transform_x=transform_x,
                                               transform_y=transform_y)
 
