@@ -5,10 +5,9 @@ import matplotlib.lines as lines
 import numpy as np
 import pandas as pd
 
-import atlas_mpl_style as ampl
-
 from quickstats.plots.template import single_frame, parse_styles, format_axis_ticks, create_transform
 from quickstats.plots import AbstractPlot
+from quickstats.utils.common_utils import combine_dict
 
 class UpperLimit1DPlot(AbstractPlot):
     
@@ -38,17 +37,14 @@ class UpperLimit1DPlot(AbstractPlot):
         self.label_map = label_map
         self.line_below = line_below
         
-        if labels is None:
-            self.labels = self.LABELS
-        else:
-            self.labels = labels        
+        self.labels = combine_dict(self.LABELS, labels)  
     
     def draw(self, logx:bool=False, xlabel:Optional[str]=None, markersize:float=50.,
              draw_observed:bool=True, draw_stat:bool=False, sig_fig:int=2):
         if (draw_observed + draw_stat) != 1:
             raise RuntimeError("either draw_observed or draw_stat should be set to True")
         n_category = len(self.category_df.columns)
-        ax = single_frame(logx=logx, styles=self.styles, analysis_label_options=self.analysis_label_options)
+        ax = self.draw_frame(logx=logx)
         transform = create_transform(transform_x='axis', transform_y='data')
         
         if draw_observed:
